/*
  ########## gLogger Mini ##########
    
	Von Martin Matysiak
		mail@k621.de
		www.k621.de

	Version 1.10

  ########## Changes: ##########

  Version 1.1 (08.01.2010):
    * Bugfix: nofs_init scans the whole SD-card for 0x03 (ETX) instead of only
      the first sector -> Existing data won't be overwritten anymore (thanks to
      Ralf Fischer for detecting and solving the problem!)

  ########## nderungen: ##########

  Version 1.1 (08.01.2010):
    * Bugfix: nofs_init durchsucht nun die gesamte SD-Karte nach 0x03 (ETX) an-
      stelle nur des ersten Sektors -> Bereits vorhandene Aufnahmen werden nicht
      mehr berschrieben (Vielen Dank an Ralf Fischer fr Erkennung und Lsung
      des Problems!)
  
  ########## License: ##########

  Copyright (c) 2010 Martin Matysiak

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

  If you like my software, I would appreciate if you could send me a postcard.
  You'll find my current address at www.k621.de -> "Impressum"

  ########## Lizenz: ##########

  Copyright (c) 2010 Martin Matysiak

  Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software und der 
  zugehrigen Dokumentationen (die "Software") erhlt, die Erlaubnis erteilt, 
  uneingeschrnkt zu benutzen, inklusive und ohne Ausnahme, dem Recht, sie zu 
  verwenden, kopieren, ndern, fusionieren, verlegen, verbreiten, unterlizenzieren 
  und/oder zu verkaufen, und Personen, die diese Software erhalten, diese Rechte 
  zu geben, unter den folgenden Bedingungen:

  Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in allen Kopien 
  oder Teilkopien der Software beizulegen.

  DIE SOFTWARE WIRD OHNE JEDE AUSDRCKLICHE ODER IMPLIZIERTE GARANTIE 
  BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FR DEN VORGESEHENEN 
  ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTSVERLETZUNG, JEDOCH NICHT 
  DARAUF BESCHRNKT. IN KEINEM FALL SIND DIE AUTOREN ODER COPYRIGHTINHABER FR 
  JEGLICHEN SCHADEN ODER SONSTIGE ANSPRCHE HAFTBAR ZU MACHEN, OB INFOLGE DER 
  ERFLLUNG EINES VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER 
  SOFTWARE ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.

  Wenn dir diese Software gefllt, wrde ich mich ber eine Postkarte von dir sehr 
  freuen. Meine aktuelle Adresse findest du unter www.k621.de -> "Impressum"

*/

#include <avr/sleep.h>
#include "global.h"
#include "modules/nofs.h"
#include "modules/gps.h"

int main (void) {

	// Interrupts global aktivieren
  sei();  

	// Ports definieren
	IO_CONF |= (1 << LED_STAT);
	LEDCODE_ON();

  // Unntige Module ausschalten (Power-reduction)
  PRR |= (1 << PRTWI) | (1 << PRTIM2) | (1 << PRTIM0) | (1 << PRTIM1) | (1 << PRADC);

  _delay_s(1);

	// Module initalisieren
  if (!(nofs_init() && gps_init())) {
    while(1) {
      _delay_ms(100);
      LEDCODE_BLINK();    
    }
  }

  LEDCODE_OFF();

  char nmeaBuf[128];

  while(1) {
    // Geschrieben wird nur, wenn NMEA-Kommando einen Fix hatte
    if(gps_getNmeaSentence(nmeaBuf, 128)) {
      nofs_writeString(nmeaBuf);
    }

    // Blinken !!! (das Wichtigste in diesem ganzen Code)
    LEDCODE_BLINK();
    _delay_ms(30);
    LEDCODE_BLINK(); 
    sleep_mode();

  }
 
	// Wird nie erreicht
	return 0;
}
