/**
  ########## gLogger Mini ##########
    
	Von Martin Matysiak
	    mail@k621.de
      www.k621.de

  Befehlsbibliothek fr das SPI-Protkoll

  ########## Licensing / Lizenz ##########

  Please take a look at the file "gLogger.c" for detailed information.

  Bitte werfen Sie einen Blick auf die Datei "gLogger.c" fr nhere
  Informationen.
*/

#include "protocols/spi.h"

uint8_t spi_init() {
  
  // SPI-Port konfigurieren:
  //  SCK, CS, MOSI als Ausgang
  //  MISO als Eingang, mit aktiviertem Pull-Up
  //  CS Standardmig High

  //  Portrichtungen definieren
  SPI_PORT_DIR |= (1 << SPI_SCK) | (1 << SPI_CS) | (1 << SPI_MOSI);
  SPI_PORT_DIR &= ~(1 << SPI_MISO);

  // Pull-Up auf MISO Leitung
  SPI_PORT |= (1 << SPI_MISO);

  // Chipselect lschen
  CLEAR_CS();
  
  // Kurz warten
  _delay_ms(10);

  // SPI-Register konfigurieren

  // SPI Master Mode, ohne Interrupts (Blocking mode)
  // MSB Zuerst
  // CPOL Mode 0 (CPOL=0, CPHA=0)
  // F_SPI = F_CPU / 128
  SPCR = (1 << SPE) | (1 << MSTR) | (1 << SPR1) | (1 << SPR0);

  return TRUE;
}

void spi_writeByte(uint8_t pByte) {
  SPDR = pByte;

  // Auf Transferergebnis warten
  while (!(SPSR & (1 << SPIF))) {
    // Energie verbrennen
  }
}

uint8_t spi_readByte() {
  // Dummybyte "senden", damit SCK-Impulse generiert werden
  SPDR = 0xFF;  

  // Auf Transferergebnis warten
  while (!(SPSR & (1 << SPIF))) {
    // Energie verbrennen
  }
    
  return SPDR;
}
